@@ -4,4 +4,4 @@ use Pixie::Store::DBI;
 
 my($spec, $user, $pass) = @ARGV;
 
-Pixie::Store::DBI->deploy($spec, $user, $pass);
+Pixie::Store::DBI->deploy($spec, user => $user, pass => $pass);
